/*
 *	keymap.c
 */

#include "filer.h"

static const KEYMAP	aDefaultFilerKeyMap[] = {
	{ // SM_HORZ
		KF_PREVPAGE, KF_NEXTPAGE, KF_PREVITEM, KF_NEXTITEM,
		KF_OK, KF_BACK, 0, 0, KF_OPTIONS, 0,
	},
	{ // SM_VERTRIGHT
		KF_PREVITEM, KF_NEXTITEM, KF_NEXTPAGE, KF_PREVPAGE,
		KF_OK, KF_BACK, 0, 0, KF_OPTIONS, 0,
	},
	{ // SM_VERTLEFT
		KF_NEXTITEM, KF_PREVITEM, KF_PREVPAGE, KF_NEXTPAGE,
		KF_OK, KF_BACK, 0, 0, KF_OPTIONS, 0,
	},
};

static const KEYMAP	aDefaultTextKeyMap[] = {
	{ // SM_HORZ
		KF_PREVPAGE, KF_NEXTPAGE, KF_PREVPAGE, KF_NEXTPAGE,
		KF_NEXTPAGE, KF_BACK, KF_BOOKMARK, KF_PREVPAGE, KF_OPTIONS, 0,
	},
	{ // SM_VERTRIGHT
		KF_PREVPAGE, KF_NEXTPAGE, KF_NEXTPAGE, KF_PREVPAGE,
		KF_NEXTPAGE, KF_BACK, KF_BOOKMARK, KF_PREVPAGE, KF_OPTIONS, 0,
	},
	{ // SM_VERTLEFT
		KF_NEXTPAGE, KF_PREVPAGE, KF_PREVPAGE, KF_NEXTPAGE,
		KF_NEXTPAGE, KF_BACK, KF_BOOKMARK, KF_PREVPAGE, KF_OPTIONS, 0,
	},
};

typedef struct _KEYFUNC {
	u8	*name;
	int	keyfunc;
} KEYFUNC, *PKEYFUNC;

static const KEYFUNC	aFilerKeyFunc[] = {
	{ "PrevPage",	KF_PREVPAGE },
	{ "NextPage",	KF_NEXTPAGE },
	{ "PrevItem",	KF_PREVITEM },
	{ "NextItem",	KF_NEXTITEM },
	{ "OK",		KF_OK },
	{ "Back",	KF_BACK },
	{ "Options",	KF_OPTIONS },
	{ NULL, NULL },
};

static const KEYFUNC	aTextKeyFunc[] = {
	{ "PrevPage",	KF_PREVPAGE },
	{ "NextPage",	KF_NEXTPAGE },
	{ "OK",		KF_OK },
	{ "Back",	KF_BACK },
	{ "Bookmark",	KF_BOOKMARK },
	{ "Options",	KF_OPTIONS },
	{ NULL, NULL },
};

#define STATIC	static

STATIC KEYMAP	aFilerKeyMap[SM_LAST];
STATIC KEYMAP	aTextKeyMap[SM_LAST];

STATIC PKEYMAP	pKeyMap;
STATIC PKEYMAP	pBaseKeyMap;

STATIC PKEYMAP	pIniKeyMap;
STATIC PKEYMAP	pIniBaseKeyMap;

STATIC PKEYFUNC	pKeyFunc;

/*
 *	GetKeyFunc
 */
int GetKeyFunc(int key)
{
	PKEYMAP		pkey;

	pkey = pKeyMap;
	if (key & KEY_LEFT)
		return pkey->left;
	else if (key & KEY_RIGHT)
		return pkey->right;
	else if (key & KEY_UP)
		return pkey->up;
	else if (key & KEY_DOWN)
		return pkey->down;
	else if (key & KEY_A)
		return pkey->a;
	else if (key & KEY_B)
		return pkey->b;
	else if (key & KEY_L)
		return pkey->l;
	else if (key & KEY_R)
		return pkey->r;
	else if (key & KEY_SELECT)
		return pkey->select;
	else if (key & KEY_START)
		return pkey->start;
	else
		return 0;
}

/*
 *	GetKeyMap
 */
PKEYMAP GetKeyMap()
{
	return pKeyMap;
}

/*
 *	SetKeyMap
 */
void SetKeyMap(int mode)
{
	pKeyMap = &pBaseKeyMap[mode];
}

/*
 *	GetFilerKeyMap
 */
PKEYMAP GetFilerKeyMap()
{
	return aFilerKeyMap;
}

/*
 *	GetTextKeyMap
 */
PKEYMAP GetTextKeyMap()
{
	return aTextKeyMap;
}

/*
 *	GetBaseKeyMap
 */
PKEYMAP GetBaseKeyMap()
{
	return pBaseKeyMap;
}

/*
 *	SetBaseKeyMap
 */
void SetBaseKeyMap(PKEYMAP pkey)
{
	pBaseKeyMap = pkey;
	SetKeyMap(GetScrnMode());
}

/*
 *	InitKeyMap
 */
void InitKeyMap()
{
	memcpy(aFilerKeyMap, aDefaultFilerKeyMap, sizeof aFilerKeyMap);
	memcpy(aTextKeyMap, aDefaultTextKeyMap, sizeof aTextKeyMap);

	SetBaseKeyMap(aFilerKeyMap);
	SetKeyMap(SM_HORZ);

	pIniKeyMap = NULL;
	pIniBaseKeyMap = NULL;
	pKeyFunc = NULL;
}

/*
 *	SetMode
 */
static int SetMode(PTOKEN ptok)
{
	int		t;

	switch (t = GetToken(ptok)) {
	default:
		t = T_ERROR;
		break;
	case T_NAME:
		if (_strnicmp(ptok->pvalue, "Filer", ptok->len) == 0)
		{
			pIniBaseKeyMap = aFilerKeyMap;
			pIniKeyMap = NULL;
			pKeyFunc = (PKEYFUNC)aFilerKeyFunc;
		}
		else if (_strnicmp(ptok->pvalue, "TextView", ptok->len) == 0)
		{
			pIniBaseKeyMap = aTextKeyMap;
			pIniKeyMap = NULL;
			pKeyFunc = (PKEYFUNC)aTextKeyFunc;
		}
		else
			t = T_ERROR;
		break;
	}

	return t;
}

/*
 *	SetDirection
 */
static int SetDirection(PTOKEN ptok)
{
	int		t;

	switch (t = GetToken(ptok)) {
	default:
		t = T_ERROR;
		break;
	case T_NAME:
		if (!pIniBaseKeyMap) {
			t = T_ERROR;
			break;
		}

		if (_strnicmp(ptok->pvalue, "Horizontal", ptok->len) == 0)
			pIniKeyMap = &pIniBaseKeyMap[SM_HORZ];
		else if (_strnicmp(ptok->pvalue, "VerticalRight", ptok->len)
			== 0)
			pIniKeyMap = &pIniBaseKeyMap[SM_VERTRIGHT];
		else if (_strnicmp(ptok->pvalue, "VerticalLeft", ptok->len)
			== 0)
			pIniKeyMap = &pIniBaseKeyMap[SM_VERTLEFT];
		else
			t = T_ERROR;
		break;
	}

	return t;
}

/*
 *	SetKey
 */
static int SetKey(PTOKEN ptok, int *pkeyfunc)
{
	int		t;
	PKEYFUNC	pf;

	switch (t = GetToken(ptok)) {
	default:
		t = T_ERROR;
		break;
	case '\n':
		break;
	case T_NAME:
		if (!pIniKeyMap || !pKeyFunc) {
			t = T_ERROR;
			break;
		}

		for (pf = pKeyFunc; pf->name; pf++) {
			if (_strnicmp(ptok->pvalue, pf->name, ptok->len) == 0)
				break;
		}
		if (!pf->name) {
			t = T_ERROR;
			break;
		}

		*pkeyfunc = pf->keyfunc;
		break;
	}

	return t;
}

/*
 *	SetKeyLeft
 */
static int SetKeyLeft(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->left);
}

/*
 *	SetKeyRight
 */
static int SetKeyRight(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->right);
}

/*
 *	SetKeyUp
 */
static int SetKeyUp(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->up);
}

/*
 *	SetKeyDown
 */
static int SetKeyDown(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->down);
}

/*
 *	SetKeyA
 */
static int SetKeyA(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->a);
}

/*
 *	SetKeyB
 */
static int SetKeyB(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->b);
}

/*
 *	SetKeyL
 */
static int SetKeyL(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->l);
}

/*
 *	SetKeyR
 */
static int SetKeyR(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->r);
}

/*
 *	SetKeySelect
 */
static int SetKeySelect(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->select);
}

/*
 *	SetKeyStart
 */
static int SetKeyStart(PTOKEN ptok)
{
	return SetKey(ptok, &pIniKeyMap->start);
}

const VALTABLE	aVTKeyMap[] = {
	{ "Mode",	SetMode },
	{ "Direction",	SetDirection },
	{ "Left",	SetKeyLeft },
	{ "Right",	SetKeyRight },
	{ "Up",		SetKeyUp },
	{ "Down",	SetKeyDown },
	{ "A",		SetKeyA },
	{ "B",		SetKeyB },
	{ "L",		SetKeyL },
	{ "R",		SetKeyR },
	{ "Select",	SetKeySelect },
	{ "Start",	SetKeyStart },
	{ NULL, NULL },
};

