/*
 *	viewjpeg.c
 */

#include "filer.h"

#include <jpeg.h>
#include <dma.h>

#define XUNIT	16
#define YUNIT	16

static int	curx, cury;
static BOOL	dirty;
static RECT	rc;

static BOOL IsJpeg(u8 *pb)
{
	return pb[0] == 0xFF && pb[1] == 0xD8;
}

static PDIRENT GetNextDirEnt(PDIRENT pdir, PDIRENT pfile)
{
	if (++pfile >= (PDIRENT)&pdir->fileaddr[pdir->namesize + pdir->filesize])
		pfile = (PDIRENT)&pdir->fileaddr[pdir->namesize];
	return pfile;
}

static PDIRENT GetPrevDirEnt(PDIRENT pdir, PDIRENT pfile)
{
	if (--pfile < (PDIRENT)&pdir->fileaddr[pdir->namesize])
		pfile = (PDIRENT)&pdir->fileaddr[pdir->namesize + pdir->filesize - sizeof(*pfile)];
	return pfile;
}

static PDIRENT NextFile(PDIRENT pdir, PDIRENT pfile)
{
	PDIRENT		pd;

	for (pd = GetNextDirEnt(pdir, pfile); pd != pfile; pd = GetNextDirEnt(pdir, pd)) {
		if (!(pd->attr & DA_DIR) && IsJpeg(&pd->fileaddr[pd->namesize]))
			break;
	}
	return pd;
}

static PDIRENT PrevFile(PDIRENT pdir, PDIRENT pfile)
{
	PDIRENT		pd;

	for (pd = GetPrevDirEnt(pdir, pfile); pd != pfile; pd = GetPrevDirEnt(pdir, pd)) {
		if (!(pd->attr & DA_DIR) && IsJpeg(&pd->fileaddr[pd->namesize]))
			break;
	}
	return pd;
}

static BOOL ViewLeft()
{
	if (curx > 0) {
		int		w;
		vu16		*pv;
		RECT		r;

		w = min(curx, XUNIT);
		curx -= w;
		Jpeg_SetPos(curx, cury, rc);

		r.x = 0;
		r.y = 0;
		r.w = w;
		r.h = rc.h;

		pv = (vu16*)VRAM_ADDR;
		MemMove16DMA3((void*)pv, (void*)&pv[w], (rc.h * rc.w - w) * 2);
		Jpeg_RefreshScreen((u16*)VRAM_ADDR, SCREEN_WIDTH, r);
	}
	return FALSE;
}

static BOOL ViewRight()
{
	if (curx < (Jpeg_GetWidth() - SCREEN_WIDTH)) {
		int		w;
		vu16		*pv;
		RECT		r;

		w = min(XUNIT, Jpeg_GetWidth() - SCREEN_WIDTH - curx);
		curx += w;
		Jpeg_SetPos(curx, cury, rc);

		r.x = rc.w - w;
		r.y = 0;
		r.w = w;
		r.h = rc.h;

		pv = (vu16*)VRAM_ADDR;
		MemCopy16DMA3((void*)&pv[w], (void*)pv, (rc.h * rc.w - w) * 2);
		Jpeg_RefreshScreen((u16*)VRAM_ADDR, SCREEN_WIDTH, r);
	}
	return FALSE;
}

static BOOL ViewUp()
{
	if (cury > 0) {
		int		h;
		vu16		*pv;
		RECT		r;

		h = min(cury, YUNIT);
		cury -= h;
		Jpeg_SetPos(curx, cury, rc);

		r.x = 0;
		r.y = 0;
		r.w = rc.w;
		r.h = h;

		pv = (vu16*)VRAM_ADDR;
		MemMove16DMA3((void*)pv,
			(void*)&pv[SCREEN_WIDTH * h],
			SCREEN_WIDTH * (SCREEN_HEIGHT - h) * sizeof(*pv));
		Jpeg_RefreshScreen((u16*)pv, SCREEN_WIDTH, r);
	}
	return FALSE;
}

static BOOL ViewDown()
{
	if (cury < (Jpeg_GetHeight() - SCREEN_HEIGHT)) {
		int		h;
		vu16		*pv;
		RECT		r;

		h = min(YUNIT, Jpeg_GetHeight() - SCREEN_HEIGHT - cury);
		cury += h;
		Jpeg_SetPos(curx, cury, rc);

		r.x = 0;
		r.y = SCREEN_HEIGHT - h;
		r.w = rc.w;
		r.h = h;

		pv = (vu16*)VRAM_ADDR;
		MemCopy16DMA3((void*)&pv[SCREEN_WIDTH * h],
			(void*)pv,
			SCREEN_WIDTH * (SCREEN_HEIGHT - h) * sizeof(*pv));
		Jpeg_RefreshScreen((u16*)pv, SCREEN_WIDTH, r);
	}
	return FALSE;
}

/*
 *	InitJpeg
 */
static void InitJpeg(PDIRENT pdir, PDIRENT pfile)
{
	ClearScrn();
	HeapInit();

	FileSys_SetFileInfo((void*)pfile, 0);
	if (!FileSys_fopen(0)) {
		Fix8printf(20, 80, "%s: cannot open.\n", pfile->fileaddr);
		return;
	}

	if (!StartJpeg(0)) {
		Fix8printf(20, 80, "StartJpeg: failed.\n");
		return;
	}

	// while ((key = GetKey()) == 0) ;

	rc.x = 0;
	rc.y = 0;
	rc.w = min(Jpeg_GetWidth(), SCREEN_WIDTH);
	rc.h = min(Jpeg_GetHeight(), SCREEN_HEIGHT);

	curx = cury = 0;
	dirty = TRUE;

	// FillScrn(0, 0, SCREEN_WIDTH, 8, COLOR_BLACK);
	// XfontSelectFontObject(FO_SMALL);
	// LeftPrintf(8, 0, SCREEN_WIDTH - 8, (u8*)"%s", pfile->fileaddr);
}

/*
 *	ViewJpeg
 */
BOOL ViewJpeg(PDIRENT pdir, PDIRENT pfile)
{
	int	key;

	if (!IsJpeg(&pfile->fileaddr[pfile->namesize]))
		return FALSE;

	InitJpeg(pdir, pfile);

	for (;;) {
		if (dirty) {
			// Fix8printf(0, 0, (u8*)"*");
			// Fix8printf(40, 0, (u8*)"Free:%6d", GetFreeMem());
			Jpeg_SetPos(curx, cury, rc);
			if ((key = Jpeg_RefreshScreen((u16*)VRAM_ADDR, SCREEN_WIDTH, rc)) & KEY_B)
				break;
			dirty = FALSE;
			// EraseScrn(0, 0, 8, 8);
			// Fix8printf(40, 0, (u8*)"Free:%6d", GetFreeMem());
		}

		while (!(key = GetKey()))
			;

		if (key & KEY_B)
			break;
		else if (key & KEY_A) {
			curx = cury = 0;
			dirty = TRUE;
		} else if (key & KEY_LEFT)
			dirty = ViewLeft();
		else if (key & KEY_RIGHT)
			dirty = ViewRight();
		else if (key & KEY_UP)
			dirty = ViewUp();
		else if (key & KEY_DOWN)
			dirty = ViewDown();
		else if (key & KEY_L) {
			pfile = PrevFile(pdir, pfile);
			InitJpeg(pdir, pfile);
		} else if (key & KEY_R) {
			pfile = NextFile(pdir, pfile);
			InitJpeg(pdir, pfile);
		}
	}

	return TRUE;

	// PrintFreeMem();
}

