/*
 *	jpeg.c
 */

#include <gba.h>

#include "jpeglib.h"

#include "jpeg_inside.h"

//===

//===

static s32 BaseX,BaseY;
static bool InitialDrawFlag;
static u32 DecodedMCUY;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

b8 LoadJpeg(int FileHandle)
{
  BaseX=0;
  BaseY=0;
  InitialDrawFlag=True;
  
  DecodedMCUY=0;
  
  pHuffmanTree=(THuffmanTree*)HeapAlloc(4*sizeof(THuffmanTree));
  pQuantizeTable=(TQuantizeTable*)HeapAlloc(4*sizeof(TQuantizeTable));
  
  if((pHuffmanTree==NULL)||(pQuantizeTable==NULL)){
    _consolePrintf("malloc HuffmanTree or QuantizeTable out of memory.\n");
    return(False);
  }
  
  rfsInit(FileHandle,0);
  rfsStartFrame();
  
  if(AnalizeJpegMarker()==False){
    _consolePrintf("JpegHeader AnalizeError.\n");
    return(False);
  }
  
  rfsClose();
  
  Jpeg_ShowFrameInfo();
  
  rbsInit(FileHandle,BitStreamStartPosition);
  
  if(Jpeg_InsideInit(FrameInfo.MCUHeightCount,FrameInfo.Width)==False) return(False);
  
  return(True);
}

b8 StartJpeg(int FileHandle)
{
  FreeJpeg();
  
  if(LoadJpeg(FileHandle)==False){
    _consolePrintf("Jpeg LoadError.\n");
    FreeJpeg();
    return(False);
  }
  
  PrintFreeMem();
  
  return(True);
}

void FreeJpeg(void)
{
  rfsClose();
  
  if(pHuffmanTree!=NULL){
    safefree(pHuffmanTree); pHuffmanTree=NULL;
  }
  
  if(pQuantizeTable!=NULL){
    safefree(pQuantizeTable); pQuantizeTable=NULL;
  }
  
  Jpeg_InsideFree();
}

// -----------------------------------------------------

void Jpeg_ShowFrameInfo(void)
{
  _consolePrintf("MCUCount (%d*%d)=%d\n",FrameInfo.MCUWidthCount,FrameInfo.MCUHeightCount,FrameInfo.MCUCount);
  _consolePrintf("PicSize (%d*%d)=%d\n",FrameInfo.Width,FrameInfo.Height,FrameInfo.Width*FrameInfo.Height);
  
  _consolePrintf("MaxFact H=%d F=%d\n",FrameInfo.SampleMaxHFact,FrameInfo.SampleMaxVFact);
  _consolePrintf("DRI %d\n",DelayRestartInterval);
  
  _consolePrintf("BitStreamStartPosition=%d\n",BitStreamStartPosition);
}

int Jpeg_RefreshScreen(u16 *pBuf,u32 BufWidth,RECT ClientRect)
{
  u32 y;
  int key;
  extern int GetKey();

  InitDecodeImage();

  LoadIWRAMMappedCode(YUVMode);
  
  if(16<=BaseY){
    u32 ReqReadyMCUY;
    
    ReqReadyMCUY=BaseY/16;
    
    if((DecodedMCUY+1)<ReqReadyMCUY){
      s32 PrgBase=DecodedMCUY+1;
      MWin_ProgressShow("SkipImage",ReqReadyMCUY-PrgBase);
      for(y=DecodedMCUY+1;y<ReqReadyMCUY;y++){
        MWin_ProgressSetPos(y-PrgBase);
        DrawImageOneLine(y,LCBM_False);
      }
      DecodedMCUY=ReqReadyMCUY-1;
      MWin_ProgressHide();
    }
  }
  
  u32 ycnt,ylimit;
  
  ylimit=(ClientRect.h/16)+2;
  // ylimit=((ClientRect.h+15)/16);
  
  if(InitialDrawFlag==True) MWin_ProgressShow("Decode...",ylimit);
  
  for(ycnt=0;ycnt<ylimit;ycnt++){
    u16 *drawbm;
    u32 srcypos;
    
    // srcypos=(BaseY/16)+ycnt;
    srcypos=((BaseY+ClientRect.y)/16)+ycnt;
    
    // if (srcypos<LCBMCount)
    if (srcypos < FrameInfo.MCUHeightCount) {
      u32 cachednum=LCBM_FindCache(srcypos);
      
      if(cachednum!=LCBM_False){
        LCBM[cachednum].Life=LCBMLifeCur; LCBMLifeCur++;
        drawbm=LCBM[cachednum].BM;
      }else{
        u32 freenum=LCBM_FindFree();

        
        if(freenum!=LCBM_False){
          if(LCBM[freenum].BM!=NULL){
            _consolePrintf("JpegPlugin InsideError. malloc LCBM but NULL.\n");
            ShowLogHalt();
          }
          LCBM[freenum].BM=(u16*)HeapAlloc(LCBMWidth*LCBMHeight*2);
          if(LCBM[freenum].BM==NULL){
            freenum=LCBM_FindOldCache();
          }
        }else{
          freenum=LCBM_FindOldCache();
        }
        
        LCBM[freenum].Life=LCBMLifeCur; LCBMLifeCur++;
        LCBM[freenum].ypos=srcypos;
        if(srcypos<FrameInfo.MCUHeightCount){
          DrawImageOneLine(srcypos,freenum);
        }
        if(DecodedMCUY<srcypos) DecodedMCUY=srcypos;
        drawbm=LCBM[freenum].BM;
      }
    }else{
      drawbm=NULL;
    }
    if(drawbm!=NULL){
      u16 *srcbm,*dstbm;
      u32 y;
      
      for(y=0;y<LCBMHeight;y++){
        s32 yofs;
        s32 base;
        
        //yofs=0-(BaseY % 16);
        yofs=0-((BaseY+ClientRect.y) % 16);
        base=(ycnt*LCBMHeight)+y+yofs;
        
        if((0<=base)&&(base<ClientRect.h)){
          //srcbm=&drawbm[BaseX+(y*LCBMWidth)];
          srcbm=&drawbm[BaseX+ClientRect.x+(y*LCBMWidth)];
          dstbm=&pBuf[ClientRect.x+((ClientRect.y+base)*BufWidth)];
          MemCopy16DMA3(srcbm,dstbm,ClientRect.w*2);
	  if ((key = GetKey()) & KEY_B)
	    return key;
        }
      }
    }
    
    if(InitialDrawFlag==True) MWin_ProgressSetPos(ycnt);
  }
  
  if(InitialDrawFlag==True){
    MWin_ProgressHide();
    InitialDrawFlag=False;
  }

  return 0;
}

void Jpeg_SetPos(s32 x,s32 y,RECT ClientRect)
{
  BaseX=x;
  if(((s32)FrameInfo.Width-ClientRect.w)<BaseX) BaseX=(s32)FrameInfo.Width-ClientRect.w;
  if(BaseX<0) BaseX=0;
  
  BaseY=y;
  if(((s32)FrameInfo.Height-ClientRect.h)<BaseY) BaseY=(s32)FrameInfo.Height-ClientRect.h;
  if(BaseY<0) BaseY=0;
}

s32 Jpeg_GetPosX(void)
{
  return(BaseX);
}

s32 Jpeg_GetPosY(void)
{
  return(BaseY);
}

s32 Jpeg_GetWidth(void)
{
  return(FrameInfo.Width);
}

s32 Jpeg_GetHeight(void)
{
  return(FrameInfo.Height);
}

// ---------------------------------------------------------------

